<?php

namespace app\abutment\logic;

use app\abutment\model\SupplierUser as SupplierUserModel;
use app\abutment\model\SupplierRelationUser as SupplierRelationUserModel;
use app\abutment\model\SupplierUser;
use think\Exception;
use think\facade\Cache;
use think\facade\Config;
use think\facade\Db;
use think\helper\Str;


//供应商账号管理
class Account
{

    //登录
    public static function login(array $param = [])
    {

        $db = new SupplierUserModel();

        $res = $db->where([
            'is_del' => $db::$is_del_normal,
            'mobile' => $param['mobile']
        ])
            ->findOrEmpty()
            ->toArray();

        if (empty($res)) return json_show(1004, '该账号不存在');
        if ($res['status'] == $db::$status_disabled) return json_show(1004, '该账号已禁用');

        $password = get_encryption_password($param['password'], $res['salt']);
        if ($password['password'] != $res['password']) return json_show(1004, '密码错误');

        //更新token
        $token = Str::random(40, -1);
        $expire_time = date('Y-m-d H:i:s', time() + mt_rand(3600, 7200));

        $rs = $db
            ->where($db->getPk(), $res[$db->getPk()])
            ->save(['token' => $token, 'expire_time' => $expire_time]);

        if (!$rs) throw new Exception('更新账号token信息失败');

        $list = SupplierRelationUserModel::field('id,supplierNo,supplierName')
            ->where([
                'is_del' => $db::$is_del_normal,
                'uid' => $res['uid'],
            ])
            ->select()
            ->toArray();

        $info = [
            'uid' => $res['uid'],
            'nickname' => $res['nickname'],
            'mobile' => $res['mobile'],
            'email' => $res['email'],
            'token' => $token,
            'expire_time' => $expire_time,
            'supplier_list' => $list,
        ];

        Cache::set(Config::get('redis_key.user_info_token') . $token, json_encode($info), $expire_time);

        return json_show(0, '登录成功', $info);


    }

    //获取供应商下普通账号列表
    public static function geAccounttList(array $param = [])
    {

        $db = new SupplierRelationUserModel();

        $rs = $db
            ->alias('a')
            ->leftJoin('supplier_user b', 'b.uid=a.uid AND b.is_del=0')
            ->where([
                'a.is_del' => $db::$is_del_normal,
                'a.supplierNo' => $param['supplierNo'],
            ]);

        if ($param['keyword'] != '') $rs->whereLike('b.nickname|b.mobile', '%' . $param['keyword'] . '%');

        $count = $rs->count('a.id');

        $list = $rs
            ->field('a.id,b.uid,b.nickname,b.mobile,b.email,a.status')
            ->order('a.addtime', 'desc')
            ->page($param['page'], $param['size'])
            ->select()
            ->toArray();

        return json_show(0, '获取成功', ['count' => $count, 'list' => $list]);

    }

    //修改密码
    public static function changePassword(array $param = [])
    {

        $db = new SupplierUserModel();

        $res = $db
            ->where([
                'uid' => $param['uid'],
                'is_del' => $db::$is_del_normal,
            ])
            ->field('uid,password,salt,status,is_del')
            ->findOrEmpty()
            ->toArray();

        if (empty($res)) return json_show(1004, '该账号不存在');
        if ($res['status'] == $db::$status_disabled) return json_show(1004, '该账号已禁用');

        $old_password = get_encryption_password($param['old_password'], $res['salt']);
        if ($old_password['password'] != $res['password']) return json_show(1004, '密码错误');

        //更新密码
        $password = get_encryption_password($param['new_password']);

        $rs = $db
            ->where($db->getPk(), $res[$db->getPk()])
            ->save(['password' => $password['password'], 'salt' => $password['salt']]);

        return $rs ? json_show(0, '修改密码成功') : json_show(1005, '修改密码失败');

    }

    //获取账号信息
    public static function readAccount(array $param = [])
    {
        $res = SupplierUser::field('uid,nickname,mobile,email')
            ->where([
                'uid' => $param['uid'],
                'is_del' => SupplierUser::$is_del_normal,
            ])
            ->append(['supplier_list'])
            ->withAttr('supplier_list', function ($val, $data) {

                return SupplierRelationUserModel::field('id,supplierNo,supplierName,status,addtime')
                    ->where([
                        'is_del' => SupplierUser::$is_del_normal,
                        'uid' => $data['uid']
                    ])
                    ->select()
                    ->toArray();
            })
            ->findOrEmpty()
            ->toArray();

        return json_show(0, '获取账号信息成功', $res);

    }

    //添加供应商账号
    public static function addAccount(array $param = [])
    {

        Db::connect('mysql_sys')->startTrans();

        try {

            $db = new SupplierUserModel();

            $res = $db
                ->field('uid,status')
                ->where([
                    'is_del' => $db::$is_del_normal,
                    'mobile' => $param['mobile'],
                ])->findOrEmpty();

            $is_insert = true;
            if ($res->isEmpty()) {
                //新增账号
                $password = get_encryption_password(Config::get('app.default_password'));
                $res->uid = $db->insertGetId([
                    'nickname' => $param['nickname'],//姓名
                    'mobile' => $param['mobile'],//手机号
                    'email' => $param['email'],//邮箱
                    'password' => $password['password'],//密码密文
                    'salt' => $password['salt'],//盐值
                    'status' => $db::$status_normal,
                    'is_del' => $db::$is_del_normal,
                ]);

                $res->status = $db::$status_normal;

            } else {

                $rs = SupplierRelationUserModel::field('id')
                    ->where([
                        'is_del' => $db::$is_del_normal,
                        'uid' => $res->uid,
                        'supplierNo' => $param['supplierNo'],
                    ])->findOrEmpty()
                    ->isEmpty();

                if (!$rs) $is_insert = false;//该关联关系已存在，无需新增

            }

            if ($is_insert) {
                SupplierRelationUserModel::create([
                    'uid' => $res->uid,
                    'supplierNo' => $param['supplierNo'],
                    'supplierName' => $param['supplierName'],
                    'status' => $res->status,
                    'is_del' => $db::$is_del_normal,
                ])->save();
            }

            Db::connect('mysql_sys')->commit();
            return json_show(0, '操作成功');
        } catch (Exception $exception) {
            Db::connect('mysql_sys')->rollback();
            return json_show(1005, '操作失败' . $exception->getMessage());
        }

    }

    //修改供应商账号
    public static function editAccount(array $param = [])
    {

        Db::connect('mysql_sys')->startTrans();

        try {

            $db = new SupplierUserModel();

            $res = $db
                ->where('is_del', $db::$is_del_normal)
                ->whereIn('mobile', [$param['mobile'], $param['old_mobile']])
                ->column('uid,status', 'mobile');

            $is_insert = true;
            if (!isset($res[$param['mobile']])) {
                //新增账号
                $password = get_encryption_password(Config::get('app.default_password'));
                $res[$param['mobile']]['uid'] = $db->insertGetId([
                    'nickname' => $param['nickname'],//姓名
                    'mobile' => $param['mobile'],//手机号
                    'email' => $param['email'],//邮箱
                    'password' => $password['password'],//密码密文
                    'salt' => $password['salt'],//盐值
                    'status' => $db::$status_normal,
                    'is_del' => $db::$is_del_normal,
                ]);

                $res[$param['mobile']]['status'] = $db::$status_normal;

            } else {

                $rs = SupplierRelationUserModel::field('id')
                    ->where([
                        'is_del' => $db::$is_del_normal,
                        'uid' => $res[$param['mobile']]['uid'],
                        'supplierNo' => $param['supplierNo'],
                    ])->findOrEmpty()
                    ->isEmpty();

                if (!$rs) $is_insert = false;//该关联关系已存在，无需新增

            }

            if ($is_insert) {
                SupplierRelationUserModel::create([
                    'uid' => $res[$param['mobile']]['uid'],
                    'supplierNo' => $param['supplierNo'],
                    'supplierName' => $param['supplierName'],
                    'status' => $res[$param['mobile']]['status'],
                    'is_del' => $db::$is_del_normal,
                ])->save();
            }
            //把旧的账号的关联关系解除
            SupplierRelationUserModel::where([
                'is_del' => $db::$is_del_normal,
                'uid' => $res[$param['old_mobile']]['uid'],
                'supplierNo' => $param['supplierNo'],
            ])->save(['is_del' => $db::$is_del_deleted]);

            Db::connect('mysql_sys')->commit();
            return json_show(0, '操作成功');
        } catch (Exception $exception) {
            Db::connect('mysql_sys')->rollback();
            return json_show(1005, '操作失败,' . $exception->getMessage());
        }

    }

    //修改供应商账号状态
    public static function statusAccount(array $param = [])
    {

        Db::connect('mysql_sys')->startTrans();

        try {

            $db = new SupplierUserModel();

            $res = $db
                ->field('uid')
                ->where([
                    'is_del' => $db::$is_del_normal,
                    'mobile' => $param['mobile'],
                ])->findOrEmpty();

            if (!$res->isEmpty()) {

                $rs = SupplierRelationUserModel::field('id')->where([
                    'is_del' => $db::$is_del_normal,
                    'uid' => $res->uid,
                    'supplierNo' => $param['supplierNo'],
                ])
                    ->where('status', '<>', $param['status'])
                    ->findOrEmpty();

                if ($rs->isEmpty()) throw new Exception('当前供应商账号未与当前供应商绑定或重复操作');
                else SupplierRelationUserModel::where('id', $rs->id)->save(['status' => $param['status']]);

            } else throw new Exception('当前供应商账号不存在');

            Db::connect('mysql_sys')->commit();
            return json_show(0, '操作成功');
        } catch (Exception $exception) {
            Db::connect('mysql_sys')->rollback();
            return json_show(1005, '操作失败,' . $exception->getMessage());
        }

    }

    //删除供应商账号
    public static function deleteAccount(array $param = [])
    {

        $where = [['is_del', '=', SupplierUserModel::$is_del_normal]];

        if (isset($param['supplierNo']) && $param['supplierNo'] != '') $where[] = ['supplierNo', '=', $param['supplierNo']];
        if (isset($param['id']) && $param['id'] != '') $where[] = ['id', '=', $param['id']];

        $res = SupplierRelationUserModel::where($where)
            ->save(['is_del' => SupplierUserModel::$is_del_deleted]);

        return $res ? json_show(0, '操作成功') : json_show(1005, '操作失败');

    }

}