<?php


namespace app\admin\controller;

//采销端管理供应商账号
use app\abutment\logic\Account as AccountLogic;
use think\facade\Config;
use think\facade\Validate;

class Account extends Base
{

    //获取供应商下普通账号列表
    public function geAccounttList()
    {

        $param = $this->request->filter('trim')->only(['supplierNo', 'keyword' => '', 'page' => 1, 'size' => 15], 'post');

        $val = Validate::rule([
            'supplierNo|供应商编码' => 'require|alphaNum|length:18',
            'keyword|关键字' => 'chsDash|max:100',
            'page|页码' => 'require|number|gt:0',
            'size|每页数量' => 'require|number|elt:100',
        ]);

        if (!$val->check($param)) return json_show(1004, $val->getError());

        return AccountLogic::geAccounttList($param);
    }

    //修改密码
    public function changePassword()
    {
        $param = $this->request->filter('trim')->only(['uid', 'old_password', 'new_password', 'confirm_password'], 'post');

        $val = Validate::rule([
            'uid|用户ID' => 'require|number|gt:0',
            'old_password|旧密码' => 'require|min:6|max:200',
            'new_password|新密码' => 'require|min:6|max:200',
            'confirm_password|确认密码' => 'require|confirm:new_password|min:6|max:200',
        ])->message(['confirm_password.confirm' => '确认密码和新密码不一致']);

        if (!$val->check($param)) return json_show(1004, $val->getError());

        return AccountLogic::changePassword($param);
    }

    //读取账号信息
    public function readAccount()
    {
        $param = $this->request->filter('trim')->only(['uid'], 'post');

        $val = Validate::rule(['uid|用户ID' => 'require|number|gt:0']);

        if (!$val->check($param)) return json_show(1004, $val->getError());

        return AccountLogic::readAccount($param);
    }

    //添加供应商账号
    public function addAccount()
    {

        $param = $this->request->filter('trim')->only(['supplierNo', 'supplierName', 'nickname', 'mobile', 'email' => ''], 'post');

        $val = Validate::rule([
            'nickname|姓名' => 'require|max:255',
            'mobile|手机号' => 'require|number|length:11|mobile',
            'email|邮箱' => 'email',
            'supplierNo|供应商编码' => 'require|alphaNum|length:18',
            'supplierName|供应商名称' => 'require|max:255',
        ]);

        if (!$val->check($param)) return json_show(1004, $val->getError());

        return AccountLogic::addAccount($param);

    }

    //修改供应商账号
    public function editAccount()
    {

        $param = $this->request->filter('trim')->only(['old_mobile', 'nickname', 'mobile', 'email'=>'','supplierNo','supplierName'], 'post');

        $val = Validate::rule([
            'nickname|姓名' => 'require|max:255',
            'mobile|手机号' => 'require|number|length:11|mobile',
            'email|邮箱' => 'email',
            'old_mobile|原始手机号' => 'require|number|length:11|mobile',
            'supplierNo|供应商编码' => 'require|alphaNum|length:18',
            'supplierName|供应商名称' => 'require|max:255',
        ]);

        if (!$val->check($param)) return json_show(1004, $val->getError());

        return AccountLogic::editAccount($param);

    }

    //启/禁用 供应商账号
    public function statusAccount()
    {
        $param = $this->request->filter('trim')->only(['supplierNo','mobile', 'status'], 'post');

        $val = Validate::rule([
            'supplierNo|供应商编码' => 'require|alphaNum|length:18',
            'mobile|手机号' => 'require|number|length:11|mobile',
            'status|状态' => 'require|number|in:' . \app\abutment\model\SupplierUser::$status_normal . ',' . \app\abutment\model\SupplierUser::$status_disabled,
        ]);

        if (!$val->check($param)) return json_show(1004, $val->getError());

        return AccountLogic::statusAccount($param);

    }

    //删除供应商账号
    public function deleteAccount()
    {
        $id = $this->request->filter('trim')->post('id/d', 0);

        return AccountLogic::deleteAccount(['id' => $id]);

    }

}